package jawful;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Locale;
import java.util.Map;

import org.junit.Test;


public class JUnitModifierTest {

	@Test
	/**
	 * Invokes ant such that it will have the compiled Jawful classes on its CP.
	 * Ant then invokes JUnit as a separate process.
	 * <p>
	 * This test confirms the resulting JUnit process runs through the classpath-shortening logic.
	 */
	public void invokeAntTest() throws IOException, InterruptedException {
		String antHome = System.getenv("ANT_HOME");
		assertNotNull("This test invokes Apache Ant and requires the environment variable ANT_HOME", antHome);
		
		File jawfulCP = new File("bin");
		File jawfulClass = new File(jawfulCP, "jawful/Wrapper.class");
		assertTrue("This test requires that Jawful is build and present in " +jawfulClass.getAbsolutePath(), jawfulClass.exists());

		String antExecutable;
		if (System.getProperty("os.name").toLowerCase(Locale.ENGLISH).contains("windows")) {
			antExecutable  = "ant.bat";
		} else {
			antExecutable  = "ant";
		}
		
		ProcessBuilder pb = new ProcessBuilder(antHome+"/bin/"+antExecutable, "-f", "testdata/build.xml", "test");
		Map<String, String> env = pb.environment();
		
		// Use this to debug the Ant process.
		//env.put("ANT_OPTS", "-agentlib:jdwp=transport=dt_socket,suspend=n,server=n,address=localhost:8000");
				
		// Ensure Ant picks up jawful at front of classpath.
		// We have to use the sys prop since the jar won't be on the classpath.
		env.put("CLASSPATH", jawfulCP.getAbsolutePath());
		env.put("ANT_OPTS", "-Djawful.jar.path="+jawfulCP.getAbsolutePath());		
		
		pb.redirectErrorStream(true);
		final Process process = pb.start();

		Pumper pumper = new Pumper(process.getInputStream());
		pumper.start();
		process.waitFor();
		pumper.join();

		assertTrue("The top stackframe element was not jawful.Wrapper",
				pumper.output.contains("----> Top callstack element is: jawful.Wrapper.main()"));
		
		assertTrue("Classpath did not contain expected element",
				pumper.output.contains("file:/C:/code/sMashBVT.workspace/jawful/testdata/manyDirectories/0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001/"));

		assertTrue("Classpath did not contain expected element",
				pumper.output.contains("file:/C:/code/sMashBVT.workspace/jawful/testdata/manyDirectories/0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000039/"));		
	
	}
	
	class Pumper extends Thread {
		String output = "";
		private InputStream is;

		public Pumper(InputStream inputStream) {
			is = inputStream;
		}

		@Override
		public void run() {
			BufferedReader reader = new BufferedReader(new InputStreamReader(is));
			String line;
			try {
				while ((line = reader.readLine()) != null) {
					System.out.println(line);
					output += line;
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
	}		
		
}
