package jawful;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.PrintStream;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Locale;

import org.junit.BeforeClass;
import org.junit.Test;

public class WrapperTest {

	private static File testDataDir = new File("testdata");
	private static File platformTestDataDir;
	
	@BeforeClass
	public static void setTestDataLocation() {
		if (System.getProperty("os.name").toLowerCase(Locale.ENGLISH).contains("windows")) {
			platformTestDataDir = new File(testDataDir, "windows");
		} else {
			platformTestDataDir = new File(testDataDir, "other");
		}
		
		assertTrue("Couldn't find testdata directory: " + platformTestDataDir.getAbsolutePath(), platformTestDataDir.exists());
	}
	
	
	@Test
	public void testReadClasspathFile() {
		Wrapper wrapper = new Wrapper();
		
		String cp = wrapper.readClasspathFile(new File(platformTestDataDir, "test1.txt").getAbsolutePath());
		final String expected1 = "a"+File.pathSeparator+"b"+File.pathSeparator+"c";
		assertEquals(expected1, cp);
		
		cp = wrapper.readClasspathFile(new File(platformTestDataDir, "test2.txt").getAbsolutePath());
		final String expected2 = "a"+File.pathSeparator+"b"+File.pathSeparator+"c";
		assertEquals(expected2, cp);
		
		cp = wrapper.readClasspathFile(new File(platformTestDataDir, "test3.txt").getAbsolutePath());
		final String expected3 = "a"+File.pathSeparator+"b"+File.pathSeparator+"c"+File.pathSeparator+"d"+File.pathSeparator+"e"+File.pathSeparator+"f";		
		assertEquals(expected3, cp);
		
		cp = wrapper.readClasspathFile(new File(platformTestDataDir, "test4.txt").getAbsolutePath());
		assertEquals("", cp);
	}

	@Test
	public void testLaunchClass() throws ClassNotFoundException, NoSuchMethodException, IllegalAccessException, InvocationTargetException, UnsupportedEncodingException {
		Wrapper wrapper = new Wrapper();
		PrintStream oldOut = System.out;
		
		ByteArrayOutputStream helloWorldOutputBytes = new ByteArrayOutputStream();
		System.setOut(new PrintStream(helloWorldOutputBytes));
		try {
			wrapper.launchClass("jawful.HelloWorld", new String[] { "I", "don't", "really", "like", "CICS"} );
		} finally {
			System.setOut(oldOut);
		}
		
		// Intentionally use default encoding
		String helloWorldOutput = new String(helloWorldOutputBytes.toByteArray(), System.getProperty("file.encoding"));
		assertTrue("Bad output: " + helloWorldOutput, helloWorldOutput.contains("----> Hello!"));
		assertTrue("Bad output: " + helloWorldOutput, helloWorldOutput.contains("CICS"));
		assertTrue("Bad output: " + helloWorldOutput, helloWorldOutput.contains("----> Goodbye!"));		
	}

	@Test
	public void testAddUrlsToSystemClassLoader() throws URISyntaxException {
		String jarToLoad1 = new File(testDataDir, "someJar1.jar").getAbsolutePath();
		String jarToLoad2 = new File(testDataDir, "someJar2.jar").getAbsolutePath();
		
		URL[] urls = ((URLClassLoader)ClassLoader.getSystemClassLoader()).getURLs();
		for (URL url : urls) {
			String cpEntry = new File(url.toURI()).getAbsolutePath();
			assertFalse("Classpath contained our test jar1 before we even loaded it!",
					cpEntry.equals(jarToLoad1));
			assertFalse("Classpath contained our test jar2 before we even loaded it!",
					cpEntry.equals(jarToLoad2));			
		}
		
		Wrapper wrapper = new Wrapper();
		wrapper.addUrlsToSystemClassLoader(jarToLoad1 + File.pathSeparator + jarToLoad2);
		
		boolean found1 = false, found2 = false;
		urls = ((URLClassLoader)ClassLoader.getSystemClassLoader()).getURLs();
		for (URL url : urls) {
			String cpEntry = new File(url.toURI()).getAbsolutePath();
			System.out.println("Checking classpath entry: " + cpEntry);
			if (cpEntry.equals(jarToLoad1)) {
				found1 = true;
			} else if (cpEntry.equals(jarToLoad2)) {
				found2 = true;
			}
		}
		assertTrue("Classpath doesn't contain our our test jars "+jarToLoad1+" and "+jarToLoad2+"!", found1 && found2);
	}

}
